from math import *


superscript = {
    "0": "\u2070",
    "1": "\u00B9",
    "2": "\u00B2",
    "3": "\u00B3",
    "4": "\u2074",
    "5": "\u2075",
    "6": "\u2076",
    "7": "\u2077",
    "8": "\u2078",
    "9": "\u2079",
    "-": "\u207B",
}


def num_to_sn_string(n: float, use_superscript = True, dp = 3) -> str:

    """
    Returns a string representation of n in scientific notation.

    The exponent can be printed in superscript characters by setting
    use_superscript to True (the default).

    Number of decimal places in the significand is set with dp (default 3).
    """

    scinot = __get_sig_exp(n)

    scinot_string = __sig_exp_to_sn_string(scinot, use_superscript, dp)

    return scinot_string


def __get_sig_exp(n: float):

    if n == 0:
        return None

    negative = n < 0

    n = abs(n)

    exponent = floor(log10(n))
    significand = n / (10**exponent)

    if negative:
        significand *= -1

    return (significand, exponent)


def __sig_exp_to_sn_string(scinot, use_superscript = True, dp = 3) -> str:

    if scinot == None:
        return "0"

    if use_superscript == True:
        exponent = __to_superscript(scinot[1])
    else:
        exponent = "^" + str(scinot[1])

    scinot_string = f"{scinot[0]:0.{dp}f} × 10{exponent}"

    return scinot_string


def __to_superscript(n: float) -> str:

    n_as_string = str(n)

    sup_string = ""

    for i in range(0, len(n_as_string)):

        sup_string += superscript[n_as_string[i]]

    return sup_string